from . cimport symbols

cpdef enum univ_pos_t:
    NO_TAG = 0
    ADJ = symbols.ADJ
    ADP
    ADV
    AUX
    CONJ
    CCONJ # U20
    DET
    INTJ
    NOUN
    NUM
    PART
    PRON
    PROPN
    PUNCT
    SCONJ
    SYM
    VERB
    X
    EOL
    SPACE
