"""
Example sentences to test spaCy and its language models.
>>> from spacy.lang.ky.examples import sentences
>>> docs = nlp.pipe(sentences)
"""

sentences = [
    "Apple Улуу Британия стартабын $1 миллиардга сатып алууну көздөөдө.",
    "Автоном автомобилдерди камсыздоо жоопкерчилиги өндүрүүчүлөргө артылды.",
    "Сан-Франциско тротуар менен жүрүүчү робот-курьерлерге тыю салууну караштырууда.",
    "Лондон - Улуу Британияда жайгашкан ири шаар.",
    "Кайдасың?",
    "Франциянын президенти ким?",
    "Америка Кошмо Штаттарынын борбор калаасы кайсы шаар?",
    "Барак Обама качан төрөлгөн?",
]
